/////////////////////////////////////////////////////////////////////////////
// Name:        Menu.cpp
// Purpose:     The class to store a DVD Menu
// Author:      Alex Thuering
// Created:		04.11.2003
// RCS-ID:      $Id: Menu.cpp,v 1.25 2004/08/01 13:07:34 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "Menu.h"
#include <wx/tokenzr.h>
#include <wxVillaLib/SConv.h>
#include <wxXML/xmlhelpr.h>

unsigned int const BUTTON_MAX_COLORS = 5;

//////////////////////////////// MenuObject //////////////////////////////////

MenuObject::MenuObject(MenuObjectType type)
{
  m_type = type;
  m_fixedSize = false;
}

void MenuObject::SetRect(const wxRect& rect)
{
  m_rect = rect;
  m_fixedSize = true;
}

void MenuObject::SetPosition(int x, int y)
{
  m_rect.x = x;
  m_rect.y = y;
}

wxBitmap MenuObject::GetBitmap(float scale)
{
  int w = (int)(GetWidth()*scale);
  int h = (int)(GetHeight()*scale);
  wxBitmap bmp(w, h);
  wxMemoryDC dc;
  dc.SelectObject(bmp);
  dc.BeginDrawing();
  dc.SetUserScale(scale, scale);
  dc.SetBackground(*wxBLACK_BRUSH);
  dc.Clear();
  Draw(dc);
  dc.EndDrawing();
  dc.SelectObject(wxNullBitmap);
  return bmp;
}

wxBitmap MenuObject::GetBitmap(int maxWidth, int maxHeight)
{
  float scale = (float) maxWidth / GetWidth();
  if (scale > (float) maxHeight / GetHeight())
	scale = (float) maxHeight / GetHeight();
  return GetBitmap(scale);
}

void MenuObject::SaveGeometryTo(wxXmlNode* node)
{
  node->AddProperty(_T("x"), wxString::Format(_T("%d"), GetX()));
  node->AddProperty(_T("y"), wxString::Format(_T("%d"), GetY()));
  if (IsFixedSize())
  {
	node->AddProperty(_T("width"), wxString::Format(_T("%d"), GetWidth()));
	node->AddProperty(_T("height"), wxString::Format(_T("%d"), GetHeight()));
  }
}

bool MenuObject::LoadGeometryFrom(wxXmlNode* node)
{
  wxString val;
  long x, y;
  if (node->GetPropVal(_T("x"), &val) && val.ToLong(&x) &&
	  node->GetPropVal(_T("y"), &val) && val.ToLong(&y))
  {
	m_rect.SetX(x);
	m_rect.SetY(y);
  }
  else
	return false;
  
  long width, height;
  if (node->GetPropVal(_T("width"), &val) && val.ToLong(&width) &&
	  node->GetPropVal(_T("height"), &val) && val.ToLong(&height))
  {
	m_rect.SetWidth(width);
	m_rect.SetHeight(height);
	m_fixedSize = true;
  }
  return true;
}

///////////////////////////////// MenuImage //////////////////////////////////

MenuImage::MenuImage(wxString fname): MenuObject(moIMAGE)
{
  if (fname.length())
	SetFileName(fname);
}

void MenuImage::SetFileName(wxString fname)
{
  m_fname = fname;
  if (!m_fname.length() || !m_image.LoadFile(m_fname))
  {
    // todo: load error image
	m_image.Create(10,10);
  }
  if (!IsFixedSize())
  {
	m_rect.width = m_image.GetWidth();
	m_rect.height = m_image.GetHeight();
  }
}

void MenuImage::FixSize(int& width, int& height)
{
  if (width > height*GetWidth()/GetHeight()) 
    height = width*GetHeight()/GetWidth();
  else
    width = height*GetWidth()/GetHeight();
}

void MenuImage::Draw(wxDC& dc, MenuDrawType drawType)
{
  double scaleX, scaleY;
  dc.GetUserScale(&scaleX, &scaleY);
  int w = (int)(GetWidth()*scaleX);
  int h = (int)(GetHeight()*scaleY);
  int x = (int)(GetX()*scaleX);
  int y = (int)(GetY()*scaleY);
  dc.SetUserScale(1,1);
  int xOrg, yOrg;
  dc.GetLogicalOrigin(&xOrg, &yOrg);
  dc.SetLogicalOrigin((int)(xOrg*scaleX), (int)(yOrg*scaleY));
  dc.DrawBitmap(m_image.Scale(w,h), x, y);
  dc.SetUserScale(scaleX, scaleY);
  dc.SetLogicalOrigin(xOrg, yOrg);
}

wxBitmap MenuImage::GetBitmap(float scale)
{
  int w = (int)(GetWidth()*scale);
  int h = (int)(GetHeight()*scale);
  return m_image.Scale(w,h);
}

wxXmlNode* MenuImage::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("image"));
  SaveGeometryTo(node);
  XmlWriteValue(node, _T("filename"), GetFileName());
  return node;
}

bool MenuImage::PutXML(wxXmlNode* node)
{
  if (!LoadGeometryFrom(node))
	return false;
  SetFileName(XmlReadValue(node, _T("filename")));
  return true;
}

///////////////////////////////// MenuText ///////////////////////////////////

MenuText::MenuText(wxString text): MenuObject(moTEXT)
{
  m_text = text;
  m_colour = *wxWHITE;
  m_font = wxFont(24, wxMODERN, wxNORMAL, wxBOLD, FALSE);
  m_antialiasing = true;
  UpdateSize();
}

void MenuText::SetText(wxString text)
{
  m_text = text;
  UpdateSize();
}

void MenuText::SetFont(wxFont font)
{
  m_font = font;
  UpdateSize();
}

void MenuText::UpdateSize()
{
  wxCoord w, h;
  wxMemoryDC dc;
  wxBitmap bitmap(1,1);
  dc.SelectObject(bitmap);
  wxFont font(m_font.GetPointSize(), m_font.GetFamily(), m_font.GetStyle(),
	m_font.GetWeight(),	m_font.GetUnderlined(), m_font.GetFaceName(),
	m_font.GetEncoding());
  dc.SetFont(font);
  dc.GetTextExtent(m_text, &w, &h);
  m_rect.width = w;
  m_rect.height = h;
  dc.SelectObject(wxNullBitmap);
}

void MenuText::Draw(wxDC& dc, MenuDrawType drawType)
{
  wxFont font(m_font.GetPointSize(), m_font.GetFamily(), m_font.GetStyle(),
	m_font.GetWeight(),	m_font.GetUnderlined(), m_font.GetFaceName(),
	m_font.GetEncoding());
  if (m_antialiasing)
  {
	dc.SetFont(font);
	dc.SetTextForeground(m_colour);
	dc.SetTextBackground(wxColour());
	dc.DrawText(m_text, GetX(), GetY());
  }
  else
  {
	wxBitmap bitmap(GetWidth(), GetHeight());
	wxMemoryDC mdc;
	mdc.SelectObject(bitmap);
	mdc.BeginDrawing();
	mdc.SetBackground(*wxBLACK_BRUSH);
	mdc.Clear();
	mdc.SetFont(font);
	mdc.SetTextForeground(*wxWHITE);
	mdc.SetTextBackground(wxColour());
	mdc.DrawText(m_text, 0, 0);
	mdc.EndDrawing();
	mdc.SelectObject(wxNullBitmap);
	
	//dc.SetPen(wxPen(m_colour,1,wxSOLID));
	wxColour bgColour = *wxBLACK;
	if (m_colour == bgColour)
	  bgColour = *wxWHITE;
	wxImage img = bitmap.ConvertToImage();
	unsigned char* data = img.GetData();
	for (int y=0; y<img.GetHeight(); y++)
	for (int x=0; x<img.GetWidth(); x++)
	{
	  if (*data != 0 && *(data+1) != 0 && *(data+2) != 0)
	  {
		*data = m_colour.Red();
		*(data+1) = m_colour.Green();
		*(data+2) = m_colour.Blue();
	  }
	  else
	  {
		*data = bgColour.Red();
		*(data+1) = bgColour.Green();
		*(data+2) = bgColour.Blue();
	  }
	  //dc.DrawPoint(GetX()+x, GetY()+y);
	  data+=3;
	}
	bitmap = wxBitmap(img);
	bitmap.SetMask(new wxMask(bitmap, bgColour));
	dc.SetTextForeground(m_colour);
	dc.SetTextBackground(wxColour());
	dc.DrawBitmap(bitmap, GetX(), GetY(), true);
  }
}

wxXmlNode* MenuText::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("text"));
  SaveGeometryTo(node);
  XmlWriteValue(node, _T("text"), GetText());
  XmlWriteValue(node, _T("colour"), SConv::ToString(GetColour()));
  XmlWriteValue(node, _T("font"), SConv::ToString(GetFont()));
  return node;
}

bool MenuText::PutXML(wxXmlNode* node)
{
  if (!LoadGeometryFrom(node))
	return false;
  SetText(XmlReadValue(node, _T("text")));
  SetColour(SConv::ToColour(XmlReadValue(node, _T("colour"))));
  SetFont(SConv::ToFont(XmlReadValue(node, _T("font"))));
  return true;
}

//////////////////////////////// MenuButton //////////////////////////////////

MenuButton::MenuButton(wxString fname, wxString label): MenuObject(moBUTTON)
{
  m_fname = fname;
  m_border = 4;
  MenuText* text = new MenuText(label); text->SetColour(*wxWHITE);
  text->SetAntialiasing(false);
  m_subObjects[mbsNORMAL].Add(text);
  text = new MenuText(label); text->SetColour(*wxBLUE);
  text->SetAntialiasing(false);
  m_subObjects[mbsHIGHLIGHTED].Add(text);
  text = new MenuText(label); text->SetColour(*wxRED);
  text->SetAntialiasing(false);
  m_subObjects[mbsSELECTED].Add(text);
  SetDirection(0, -1);
  SetDirection(1, -1);
  SetDirection(2, -1);
  SetDirection(3, -1);
  UpdateSize();
}

void MenuButton::SetBackgroundColour(wxColour colour,
 MenuButtonState state)
{
  m_bgColour[state] = colour;
}

wxColour MenuButton::GetBackgroundColour(MenuButtonState state)
{
  return m_bgColour[state];
}

wxString MenuButton::GetParam(wxString name, MenuButtonState state)
{
  if (name == _T("text"))
	return ((MenuText*) m_subObjects[0][0])->GetText();
  return wxEmptyString;
}

void MenuButton::SetParam(wxString name, wxString value, MenuButtonState state)
{
  if (name == _T("text"))
  {
	((MenuText*) m_subObjects[0][0])->SetText(value);
	((MenuText*) m_subObjects[1][0])->SetText(value);
	((MenuText*) m_subObjects[2][0])->SetText(value);
  }
  UpdateSize();
}

wxFont MenuButton::GetParamFont(wxString name, MenuButtonState state)
{
  if (name == _T("text_font"))
	return ((MenuText*) m_subObjects[0][0])->GetFont();
  return wxFont();
}

void MenuButton::SetParamFont(wxString name, wxFont value, MenuButtonState state)
{
  if (!value.Ok())
	value = wxFont(24, wxMODERN, wxNORMAL, wxBOLD, FALSE);
  if (name == _T("text_font"))
  {
	((MenuText*) m_subObjects[0][0])->SetFont(value);
	((MenuText*) m_subObjects[1][0])->SetFont(value);
	((MenuText*) m_subObjects[2][0])->SetFont(value);
  }
  UpdateSize();
}

wxColour MenuButton::GetParamColour(wxString name, MenuButtonState state)
{
  if (name == _T("text_colour"))
	return ((MenuText*) m_subObjects[state][0])->GetColour();
  return wxColour();
}

void MenuButton::SetParamColour(wxString name, wxColour value, MenuButtonState state)
{
  if (name == _T("text_colour"))
	((MenuText*) m_subObjects[state][0])->SetColour(value);
}

void MenuButton::UpdateSize()
{
  m_rect.width = 0;
  m_rect.height = 0;
  for (int i=0; i<(int)m_subObjects[0].GetCount(); i++)
  {
	MenuObject* obj = m_subObjects[0][i];
	if (m_rect.width < obj->GetWidth())
	  m_rect.width = obj->GetWidth();
	if (m_rect.height < obj->GetHeight())
	  m_rect.height = obj->GetHeight();
  }
  m_rect.width += 2*m_border;
  m_rect.height += 2*m_border;
}

void MenuButton::FixSize(int& width, int& height)
{
  width = GetWidth();
  height = GetHeight();
}

void MenuButton::Draw(wxDC& dc, MenuDrawType drawType)
{
  MenuButtonState state = mbsNORMAL;
  if (drawType == mdBUTTONS_HIGHLIGHTED)
	state = mbsHIGHLIGHTED;
  else if (drawType == mdBUTTONS_SELECTED)
	state = mbsSELECTED;
  
  if (m_bgColour[state].Ok())
  {
	dc.SetPen(*wxTRANSPARENT_PEN);
	dc.SetBrush(wxBrush(m_bgColour[state], wxSOLID));
	dc.DrawRectangle(GetX(), GetY(), GetWidth(), GetHeight());
  }
  
  int xOrg, yOrg;
  dc.GetLogicalOrigin(&xOrg, &yOrg);
  for (int i=0; i<(int)m_subObjects[state].GetCount(); i++)
  {
	MenuObject* obj = m_subObjects[state][i];
	dc.SetLogicalOrigin(xOrg-GetX()-m_border, yOrg-GetY()-m_border);
	obj->Draw(dc);
  }
  dc.SetLogicalOrigin(xOrg, yOrg);
}

int MenuButton::btInd = 0;

void MenuButton::ResetButtonIndex()
{
  btInd = 0;
}

wxXmlNode* MenuButton::GetXML(DVDFileType type)
{
  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("button"));
  btInd++;
  
  switch (type)
  {
	case DVDSTYLER_XML:
	  SaveGeometryTo(node);
	  XmlWriteValue(node, _T("action"), GetAction());
	  XmlWriteValue(node, _T("type"), GetFileName());
	  XmlWriteValue(node, _T("text"), GetParam(_T("text")));
	  XmlWriteValue(node, _T("text_font"), SConv::ToString(GetParamFont(_T("text_font"))));
	  XmlWriteValue(node, _T("text_colour/normal"), SConv::ToString(GetParamColour(_T("text_colour"))));
	  XmlWriteValue(node, _T("text_colour/highlighted"), SConv::ToString(GetParamColour(_T("text_colour"), mbsHIGHLIGHTED)));
	  XmlWriteValue(node, _T("text_colour/selected"), SConv::ToString(GetParamColour(_T("text_colour"), mbsSELECTED)));
	  XmlWriteValue(node, _T("bgcolour/normal"), SConv::ToString(GetBackgroundColour()));
	  XmlWriteValue(node, _T("bgcolour/highlighted"), SConv::ToString(GetBackgroundColour(mbsHIGHLIGHTED)));
	  XmlWriteValue(node, _T("bgcolour/selected"), SConv::ToString(GetBackgroundColour(mbsSELECTED)));
	  XmlWriteValue(node, _T("direction/left"),  wxString::Format(_T("%d"), GetDirection(0)));
	  XmlWriteValue(node, _T("direction/right"), wxString::Format(_T("%d"), GetDirection(1)));
	  XmlWriteValue(node, _T("direction/up"),    wxString::Format(_T("%d"), GetDirection(2)));
	  XmlWriteValue(node, _T("direction/down"),  wxString::Format(_T("%d"), GetDirection(3)));
	  break;
	case SPUMUX_XML:
	  node->AddProperty(_T("name"), wxString::Format(_T("%d"), btInd));
	  node->AddProperty(_T("x0"), wxString::Format(_T("%d"), GetX()));
	  node->AddProperty(_T("y0"), wxString::Format(_T("%d"), GetY()));
	  node->AddProperty(_T("x1"), wxString::Format(_T("%d"), GetX() + GetWidth()));
	  node->AddProperty(_T("y1"), wxString::Format(_T("%d"), GetY() + GetHeight()));
	  if (GetDirection(0) != -1)
		node->AddProperty(_T("left"),  wxString::Format(_T("%d"), GetDirection(0)+1));
	  if (GetDirection(1) != -1)
		node->AddProperty(_T("right"), wxString::Format(_T("%d"), GetDirection(1)+1));
	  if (GetDirection(2) != -1)
		node->AddProperty(_T("up"),    wxString::Format(_T("%d"), GetDirection(2)+1));
	  if (GetDirection(3) != -1)
		node->AddProperty(_T("down"),  wxString::Format(_T("%d"), GetDirection(3)+1));
	  break;
	case DVDAUTHOR_XML:
	  node->AddProperty(_T("name"), wxString::Format(_T("%d"), btInd));
	  node->AddChild(new wxXmlNode(wxXML_TEXT_NODE, wxEmptyString, GetAction()));
	  break;
	default:
	  break;
  }
  return node;
}

bool MenuButton::PutXML(wxXmlNode* node)
{
  if (!LoadGeometryFrom(node))
	return false;
  long lval;
  if (XmlReadValue(node, _T("direction/left")).ToLong(&lval))
	SetDirection(0, lval);
  if (XmlReadValue(node, _T("direction/right")).ToLong(&lval))
	SetDirection(1, lval);
  if (XmlReadValue(node, _T("direction/up")).ToLong(&lval))
	SetDirection(2, lval);
  if (XmlReadValue(node, _T("direction/down")).ToLong(&lval))
	SetDirection(3, lval);
  SetAction(XmlReadValue(node, _T("action")));
  SetFileName(XmlReadValue(node, _T("type")));
  SetParam(_T("text"), XmlReadValue(node, _T("text")));
  SetParamFont(_T("text_font"), SConv::ToFont(XmlReadValue(node, _T("text_font"))));
  SetParamColour(_T("text_colour"), SConv::ToColour(XmlReadValue(node, _T("text_colour/normal"))));
  SetParamColour(_T("text_colour"), SConv::ToColour(XmlReadValue(node, _T("text_colour/highlighted"))), mbsHIGHLIGHTED);
  SetParamColour(_T("text_colour"), SConv::ToColour(XmlReadValue(node, _T("text_colour/selected"))), mbsSELECTED);
  SetBackgroundColour(SConv::ToColour(XmlReadValue(node, _T("bgcolour/normal"))));
  SetBackgroundColour(SConv::ToColour(XmlReadValue(node, _T("bgcolour/highlighted"))), mbsHIGHLIGHTED);
  SetBackgroundColour(SConv::ToColour(XmlReadValue(node, _T("bgcolour/selected"))), mbsSELECTED);
  return true;
}

////////////////////////////////// Menu //////////////////////////////////////

Menu::Menu(VideoFormat videoFormat)
{
  m_videoFormat = videoFormat;
  m_startTime = _T("00:00:00.00");
  m_bgColour = *wxBLACK;
}

Menu::~Menu()
{
  WX_CLEAR_ARRAY(m_objects)
}

wxArrayString Menu::GetVideoFormatStrings()
{
  wxArrayString formats;
  formats.Add(_T("PAL 720x576"));
  formats.Add(_T("NTSC 720x480"));
  return formats;
}

wxSize Menu::GetResolution()
{
  if (m_videoFormat == vfNTSC)
	return wxSize(720,480);
  return wxSize(720,576); // vfPAL
}

bool Menu::HasVideoBackground()
{
  if (GetBackground().length() &&
	  !wxImage::FindHandler(GetBackground().AfterLast(wxT('.')).Lower(), -1))
	return true;
  return false;
}

wxBitmap Menu::GetImage(MenuDrawType drawType)
{
  wxBitmap bitmap;
  if (m_bgFile.length() &&
	  (drawType == mdALL || drawType == mdBACKGROUND))
  {
	wxImage img;
	if (!img.LoadFile(m_bgFile))
	  return bitmap;
	img.Rescale(GetResolution().GetWidth(), GetResolution().GetHeight());
	bitmap = wxBitmap(img);
  }
  else
	bitmap.Create(GetResolution().GetWidth(), GetResolution().GetHeight());
  
  wxMemoryDC dc;
  dc.SelectObject(bitmap);
  dc.BeginDrawing();
  
  if (!m_bgFile.length() ||
	  (drawType != mdALL && drawType != mdBACKGROUND))
  {
	dc.SetBackground(*wxBLACK_BRUSH);
	dc.Clear();
  }
  
  for (int i=0; i<GetObjectsCount(); i++)
  {
	MenuObject* obj = GetObject(i);
	if (drawType == mdALL ||
		(drawType == mdBACKGROUND && obj->GetType() != moBUTTON) ||
		(drawType != mdBACKGROUND && obj->GetType() == moBUTTON))
	  obj->Draw(dc, drawType);
  }
  
  dc.EndDrawing();
  dc.SelectObject(wxNullBitmap);
  
  if (drawType != mdALL && drawType != mdBACKGROUND)
  {
	// count of colors must be < BUTTON_MAX_COLORS
    wxImage img = bitmap.ConvertToImage();
    if (img.CountColours(BUTTON_MAX_COLORS+1)>BUTTON_MAX_COLORS)
    {
      wxImageHistogram histogram;
      img.ComputeHistogram(histogram);
      wxArrayLong colors;
      wxArrayLong values;
      wxImageHistogram::iterator it;
      for(it = histogram.begin(); it != histogram.end(); ++it )
      {
        colors.Add(it->first);
        values.Add(it->second.value);        
      }
      //sort
      bool flag = true;
      while (flag)
      {
        flag = false;
        for (int i=0; i<(int)colors.GetCount()-1; i++)
        if (values[i] < values[i+1])
        {
          long tmp = values[i];
          values[i] = values[i+1];
          values[i+1] = tmp;
          tmp = colors[i];
          colors[i] = colors[i+1];
          colors[i+1] = tmp;
          flag = true;
        }
      }
	  //todo: remove like colors
      //create Palette
      unsigned long bg_color = colors[0]; //transparent color
      unsigned char r[BUTTON_MAX_COLORS-1],
       g[BUTTON_MAX_COLORS-1], b[BUTTON_MAX_COLORS-1];
      for (int i=0; i<(int)BUTTON_MAX_COLORS-1; i++)
      {
        unsigned long color = (unsigned long) colors[i+1];
        r[i] = (color>>16)&0xFF;
        g[i] = (color>>8)&0xFF;
        b[i] = color&0xFF;
      }
      wxPalette palette(BUTTON_MAX_COLORS-1, r, g, b);
      //img.SetPalette(palette);
      unsigned char* data = img.GetData();
      for (int y=0; y<img.GetHeight(); y++)
      for (int x=0; x<img.GetWidth(); x++)
      {
        if (*((unsigned long*)data) != bg_color)
        {
          int n = palette.GetPixel(data[0], data[1], data[2]);
          palette.GetRGB(n, data, data+1, data+2);
          //data[0] = 255; data[1] = 255; data[2] = 255;
        }
        data+=3;
      }
      bitmap = wxBitmap(img);
    }
  }
  
  return bitmap;
}

MenuImage* Menu::AddImage(wxString fname, int x, int y)
{
  MenuImage* obj = new MenuImage(fname);
  obj->SetPosition(x, y);
  m_objects.Add(obj);
  return obj;
}

MenuText* Menu::AddText(wxString text, int x, int y)
{
  MenuText* obj = new MenuText(text);
  obj->SetPosition(x, y);
  m_objects.Add(obj);
  return obj;
}

MenuButton* Menu::AddButton(wxString fname, wxString label, int x, int y)
{
  MenuButton* obj = new MenuButton(fname, label);
  obj->SetPosition(x, y);
  obj->SetAction(_T("jump title 1;"));
  m_objects.Add(obj);
  return obj;
}

bool Menu::SaveSpumux(wxString fname,
  wxString btFile, wxString hlFile, wxString selFile)
{
  wxXmlDocument xml;
  wxXmlNode* root = new wxXmlNode(wxXML_ELEMENT_NODE, _T("subpictures"));
  wxXmlNode* streamNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("stream"));
  wxXmlNode* spuNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("spu"));
  if (GetStartTime().Length())
	spuNode->AddProperty(_T("start"), GetStartTime());
  if (GetEndTime().Length())
	spuNode->AddProperty(_T("end"), GetEndTime());
  spuNode->AddProperty(_T("image"), btFile);
  spuNode->AddProperty(_T("highlight"), hlFile);
  spuNode->AddProperty(_T("select"), selFile);
  spuNode->AddProperty(_T("transparent"), _T("000000"));
  spuNode->AddProperty(_T("force"), _T("yes"));
  MenuButton::ResetButtonIndex();
  for (int i=0; i<(int)GetObjectsCount(); i++)
  {
    MenuObject* obj = GetObject(i);
	if (obj->GetType() == moBUTTON)
	  spuNode->AddChild(obj->GetXML(SPUMUX_XML));
  }
  streamNode->AddChild(spuNode);
  root->AddChild(streamNode);
  xml.SetRoot(root);
  return xml.Save(fname);
}

wxXmlNode* Menu::GetXML(DVDFileType type, wxXmlNode* node)
{
  if (node == NULL)
	node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("menu"));
  
  if (type == DVDSTYLER_XML)
  {
	node->AddProperty(_T("videoFormat"), (m_videoFormat == vfPAL) ? _T("PAL") : _T("NTSC"));
	if (m_bgFile.size())
	  node->AddProperty(_T("bgFile"), m_bgFile);
	else
	  node->AddProperty(_T("bgColour"), SConv::ToString(m_bgColour));
  }
  
  MenuButton::ResetButtonIndex();
  for (int i=0; i<GetObjectsCount(); i++)
  {
	MenuObject* obj = GetObject(i);
	if (type == DVDSTYLER_XML || obj->GetType() == moBUTTON)
	  node->AddChild(obj->GetXML(type));
  }
  return node;
}

bool Menu::PutXML(wxXmlNode* node)
{
  wxXmlNode* child = node->GetChildren();
  if (child != NULL && child->GetName() == _T("menu"))
  {
	wxString val;
	
	m_videoFormat = vfPAL;
	if (child->GetPropVal(_T("videoFormat"), &val) && val == _T("NTSC"))
	  m_videoFormat = vfNTSC;
	
	if (child->GetPropVal(_T("bgFile"), &val))
	  m_bgFile = val;
	else if (child->GetPropVal(_T("bgColour"), &val))
	  m_bgColour = SConv::ToColour(val);
	
	for (child = child->GetChildren(); child != NULL; child = child->GetNext())
	{
	  if (child->GetType() != wxXML_ELEMENT_NODE)
		continue;
	  MenuObject* obj;
	  if (child->GetName() == _T("button"))
		obj = new MenuButton();
	  else if (child->GetName() == _T("image"))
		obj = new MenuImage();
	  else if (child->GetName() == _T("text"))
		obj = new MenuText();
	  else
		continue;
	  if (!obj->PutXML(child))
		continue;
	  m_objects.Add(obj);
	}
  }
  return true;
}
