/////////////////////////////////////////////////////////////////////////////
// Name:        Menu.h
// Purpose:     The class to store a DVD Menu
// Author:      Alex Thuering
// Created:		04.11.2003
// RCS-ID:      $Id: Menu.h,v 1.14 2004/07/26 12:39:47 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef MENU_H
#define MENU_H

#include <wx/wx.h>
#include <wx/image.h>
#include <wx/dynarray.h>
#include <wxXML/xmlhelpr.h>

enum DVDFileType { DVDSTYLER_XML, DVDAUTHOR_XML, SPUMUX_XML };

enum VideoFormat
{
  vfPAL = 0,
  vfNTSC
};

enum MenuObjectType
{
  moIMAGE,
  moTEXT,
  moBUTTON
};

enum MenuDrawType
{
  mdALL,
  mdBACKGROUND,
  mdBUTTONS_NORMAL,
  mdBUTTONS_HIGHLIGHTED,
  mdBUTTONS_SELECTED
};

enum MenuButtonState
{
  mbsNORMAL = 0,
  mbsHIGHLIGHTED,
  mbsSELECTED
};

class MenuObject
{
  public:
	MenuObject(MenuObjectType type);
	virtual ~MenuObject() {} 
	
	inline MenuObjectType GetType() { return m_type; }
	
	inline wxRect GetRect() { return m_rect; }
	inline int GetX() { return m_rect.GetX(); }
	inline int GetY() { return m_rect.GetY(); }
	inline int GetWidth() { return m_rect.GetWidth(); }
	inline int GetHeight() { return m_rect.GetHeight(); }
	inline bool IsFixedSize() { return m_fixedSize; }
	void SetRect(const wxRect& rect);
	void SetPosition(int x, int y);
	virtual void FixSize(int& width, int& height) {}
	
	virtual void Draw(wxDC& dc, MenuDrawType drawType = mdALL) = 0;
	virtual wxBitmap GetBitmap(float scale = 1);
	wxBitmap GetBitmap(int maxWidth, int maxHeight);
	
	virtual wxXmlNode* GetXML(DVDFileType type) = 0;
	virtual bool PutXML(wxXmlNode* node) = 0;
	
  protected:
	MenuObjectType m_type;
	bool m_fixedSize;
	wxRect m_rect;
	void SaveGeometryTo(wxXmlNode* node);
	bool LoadGeometryFrom(wxXmlNode* node);
};

WX_DEFINE_ARRAY(MenuObject*, MenuObjects);

class MenuImage: public MenuObject
{
  public:
	MenuImage(wxString fname = _T(""));
	
	inline wxString GetFileName() { return m_fname; }
	void SetFileName(wxString fname);
	
	void FixSize(int& width, int& height);
	void Draw(wxDC& dc, MenuDrawType drawType = mdALL);
	wxBitmap GetBitmap(float scale = 1);
	
	wxXmlNode* GetXML(DVDFileType type);
	bool PutXML(wxXmlNode* node);
	
  protected:
    wxString m_fname;
	wxImage m_image;
};

class MenuText: public MenuObject
{
  public:
	MenuText(wxString text = _T(""));
	
	inline wxString GetText() { return m_text; }
	void SetText(wxString text);
	
	inline void SetColour(wxColour colour) { m_colour = colour; }
	inline wxColour GetColour() { return m_colour; }
	
	inline wxFont GetFont() { return m_font; }
	void SetFont(wxFont font);
	
	inline void SetAntialiasing(bool value) { m_antialiasing = value; }
	inline bool GetAntialiasing() { return m_antialiasing; }
	
	void Draw(wxDC& dc, MenuDrawType drawType = mdALL);
	
	wxXmlNode* GetXML(DVDFileType type);
	bool PutXML(wxXmlNode* node);
	
  protected:
    wxString m_text;
	wxColour m_colour;
	wxFont m_font;
	bool m_antialiasing;
	void UpdateSize();
};

class MenuButton: public MenuObject
{
  public:
	MenuButton(wxString fname = _T(""), wxString label = _T("button"));
	
	inline wxString GetFileName() { return m_fname; }
	inline void 	SetFileName(wxString fname) { m_fname = fname; }
	
	inline wxString GetAction() { return m_action; }
	inline void 	SetAction(wxString action) { m_action = action; }
	
	int 	 GetDirection(int dir) { return m_direction[dir]; }
	void 	 SetDirection(int dir, int value) { m_direction[dir] = value; }
	
	void 	 SetBackgroundColour(wxColour colour, MenuButtonState state = mbsNORMAL);
	wxColour GetBackgroundColour(MenuButtonState state = mbsNORMAL);
	
	wxString GetParam(wxString name, MenuButtonState state = mbsNORMAL);
	void 	 SetParam(wxString name, wxString value, MenuButtonState state = mbsNORMAL);
	wxFont 	 GetParamFont(wxString name, MenuButtonState state = mbsNORMAL);
	void	 SetParamFont(wxString name, wxFont value, MenuButtonState state = mbsNORMAL);
	wxColour GetParamColour(wxString name, MenuButtonState state = mbsNORMAL);
	void 	 SetParamColour(wxString name, wxColour value, MenuButtonState state = mbsNORMAL);
	
	void	 FixSize(int& width, int& height);
	void 	 Draw(wxDC& dc, MenuDrawType drawType = mdALL);
	
	static void ResetButtonIndex();
	wxXmlNode* GetXML(DVDFileType type);
	bool PutXML(wxXmlNode* node);
	
  protected:
	static int  btInd;
    wxString 	m_fname;
	wxString 	m_action;
	int 	 	m_direction[4]; // left, right, up, down button names
	int 	 	m_border;
	wxColour	m_bgColour[3];
	MenuObjects m_subObjects[3];
	void 	 	UpdateSize();
};

class Menu
{
  public:
	Menu(VideoFormat videoFormat = vfPAL);
	~Menu();
	
	static wxArrayString GetVideoFormatStrings();
	inline VideoFormat   GetVideoFormat() { return m_videoFormat; }
	inline void SetVideoFormat(VideoFormat format) { m_videoFormat = format; }
	wxSize GetResolution();
	
	inline wxString GetStartTime() { return m_startTime; }
	inline void 	SetStartTime(wxString time) { m_startTime = time; }
	inline wxString GetEndTime() { return m_endTime; }
	inline void 	SetEndTime(wxString time) { m_endTime = time; }
	
	inline void 	SetBackground(wxString fname) { m_bgFile = fname; }
	inline wxString GetBackground() { return m_bgFile; }
	inline void 	SetBackgroundColour(wxColour colour) { m_bgColour = colour; }
	inline wxColour GetBackgroundColour() { return m_bgColour; }
	bool			HasVideoBackground();
	
	MenuImage*  	AddImage(wxString fname, int x = 0, int y = 0);
	MenuText*   	AddText(wxString text, int x = 0, int y = 0);
	MenuButton* 	AddButton(wxString fname, wxString label, int x = 0, int y = 0);
	
	int				GetObjectsCount() { return m_objects.Count(); }
	MenuObject* 	GetObject(int index) { return m_objects.Item(index); }
	
	inline void 	removeObject(int index) { m_objects.RemoveAt(index); }
	
	wxBitmap 		GetImage(MenuDrawType drawType = mdALL);
	
	bool 			SaveSpumux(wxString fname,  wxString btFile, wxString hlFile, wxString selFile);
	wxXmlNode*		GetXML(DVDFileType type, wxXmlNode* node = NULL);
	bool			PutXML(wxXmlNode* node);


  protected:
	VideoFormat m_videoFormat;
	wxString 	m_startTime, m_endTime;
	wxString 	m_bgFile;
	wxColour 	m_bgColour;
	MenuObjects m_objects;
};

#endif // MENU_H
