/////////////////////////////////////////////////////////////////////////////
// Name:        PropDlg.cpp
// Purpose:     Properties dialog (base class for all properties dialogs)
// Author:      Alex Thuering
// Created:     18.11.2003
// RCS-ID:      $Id: PropDlg.cpp,v 1.7 2004/04/18 09:58:18 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////
 
#include "PropDlg.h"
#include <wx/fontdlg.h>
#include <wx/colordlg.h>
#include <wx/grid.h>

//////////////////////////////////////////////////////////////////////////////
//////////////////////////// ColourPanel ////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

class ColourPanel: public wxPanel
{
  public:
	ColourPanel(wxWindow* parent, wxWindowID id): wxPanel(parent, id,
	  wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL)
	{
	  m_bgColour = GetBackgroundColour();
	}
	
	inline void SetColour(wxColour colour)
	{
	  m_colour = colour;
	  if (colour.Ok())
		SetBackgroundColour(colour);
	  else
		SetBackgroundColour(m_bgColour);
	  Refresh();
	}
	
	inline wxColour GetColour()	{ return m_colour; }
	
  protected:
	wxColour m_colour;
	wxColour m_oldColour;
	wxColour m_bgColour;
	
	void OnPaint(wxPaintEvent &event)
	{
	  wxPaintDC dc(this);  
	  PrepareDC(dc);
	  if (!m_colour.Ok())
	  {
		dc.BeginDrawing();
		dc.SetPen(*wxBLACK_PEN);
		dc.SetBrush(*wxTRANSPARENT_BRUSH);
		int w = GetClientSize().GetWidth();
		int h = GetClientSize().GetHeight();
		dc.DrawLine(0, 0, w, h);
		dc.DrawLine(w, 0, 0, h);
		dc.EndDrawing();
	  }
	}
	
	void OnClick(wxMouseEvent &event)
	{
	  if (m_colour.Ok())
	  {
		m_oldColour = GetColour();
		SetColour(wxColour());
	  }
	  else
		SetColour(m_oldColour);
	  Refresh();
	}
	
  private:
	DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(ColourPanel, wxPanel)
  EVT_PAINT(ColourPanel::OnPaint)
  EVT_LEFT_DOWN(ColourPanel::OnClick)
END_EVENT_TABLE()

//////////////////////////////////////////////////////////////////////////////
/////////////////////////////// PropDlg //////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

enum
{
  RESET_BT_ID = 7830,
  SELECT_FONT_BT_ID,
  SELECT_COLOR_BT_ID,
  GROUP_CHECK_ID,
  ROW_DELETE_ID
};

BEGIN_EVENT_TABLE(PropDlg, wxDialog)
  EVT_BUTTON(wxID_CANCEL,		 PropDlg::OnCancel)
  EVT_BUTTON(wxID_OK,			 PropDlg::OnOk)
  EVT_BUTTON(RESET_BT_ID,		 PropDlg::OnReset)
  EVT_BUTTON(SELECT_FONT_BT_ID,  PropDlg::OnSelectFont)
  EVT_BUTTON(SELECT_COLOR_BT_ID, PropDlg::OnSelectColour)
  EVT_CHECKBOX(GROUP_CHECK_ID,   PropDlg::OnGroupCheck)
  EVT_GRID_CELL_LEFT_CLICK(PropDlg::OnCellLeftClick)
  EVT_GRID_CELL_RIGHT_CLICK(PropDlg::OnCellRightClick)
  EVT_GRID_CELL_CHANGE(PropDlg::OnCellChange)
  EVT_MENU(ROW_DELETE_ID, PropDlg::OnRowDelete)
END_EVENT_TABLE()

enum PropertyType
{
  propTEXT = 0,
  propSTAT_TEXT,
  propCOMBO,
  propCHECK,
  propGRID,
  propCOLOUR,
  propFONT
};

PropDlg::PropDlg(wxWindow* parent): wxDialog(parent, -1,
  _("Properties"), wxDefaultPosition, wxDefaultSize,
  wxDEFAULT_DIALOG_STYLE| wxRESIZE_BORDER)
{
  m_currGroupId = 0;
}

void PropDlg::Create(bool resetButton)
{
  wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer* propSizer = new wxBoxSizer(wxVERTICAL);
  CreatePropPanel(propSizer);
  sizer->Add(propSizer, 1, wxEXPAND|wxALL, 10);
  CreateButtonPane(sizer, resetButton);
  SetAutoLayout(true);
  SetSizer(sizer);
  sizer->Fit(this);
  sizer->SetSizeHints(this);
  Layout();
  Centre();
}

PropDlg::~PropDlg()
{
  for (int i=0; i<(int)m_types.GetCount(); i++)
	switch(m_types[i])
	{
	  case propFONT:
		delete (wxFontData*)m_controls[i];
		//delete ((wxFontData**)m_controls[i])[0];
		//delete[] (void**)m_controls[i];
		break;
	}
}

void PropDlg::OnCancel(wxCommandEvent &WXUNUSED(event))
{
  EndModal(wxID_CANCEL);
}

void PropDlg::OnOk(wxCommandEvent &WXUNUSED(event))
{
  if (SetValues())
    EndModal(wxID_OK);
}

void PropDlg::OnReset(wxCommandEvent& event)
{
  Reset();
}

void PropDlg::CreatePropPanel(wxSizer* sizer)
{
}

void PropDlg::CreateButtonPane(wxSizer* sizer, bool resetButton)
{
  wxBoxSizer* buttonPane = new wxBoxSizer(wxHORIZONTAL);
  buttonPane->Add(10, 10, 1, wxEXPAND);
  wxButton* okBt = new wxButton(this, wxID_OK, _("OK"));
  okBt->SetDefault();
  buttonPane->Add(okBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
  buttonPane->Add(8, 0);
  wxButton* cancelBt = new wxButton(this, wxID_CANCEL, _("Cancel"));
  buttonPane->Add(cancelBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
  if (resetButton)
  {
    buttonPane->Add(8, 0);
    wxButton* resetBt = new wxButton(this, RESET_BT_ID, _("Reset"));
    buttonPane->Add(resetBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
  }
  buttonPane->Add(10, 10, 1, wxEXPAND);
  sizer->Add(buttonPane, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);
}

void PropDlg::Reset()
{
  m_updateIndex = 0;
  CreatePropPanel(NULL);
}

void PropDlg::AddText(wxSizer* sizer, wxString text)
{
  if (!sizer)
	return;
  // add static text
  if (text.length())
	sizer->Add(new wxStaticText(this, -1, text));
  else
	sizer->Add(10,10);
}

void PropDlg::AddTextProp(wxSizer* sizer, wxString text, wxString defValue,
  bool readonly)
{
  if (!sizer) // only update value
  {
	wxTextCtrl* ctrl = (wxTextCtrl*) m_controls[m_updateIndex];
	ctrl->SetValue(defValue);
	ctrl->SetEditable(!readonly);
	m_updateIndex++;
	return;
  }
  // add text control
  if (text.length())
	sizer->Add(new wxStaticText(this, -1, text));
  wxTextCtrl* ctrl = new wxTextCtrl(this, -1, defValue);
  ctrl->SetEditable(!readonly);
  sizer->Add(ctrl, 1, wxEXPAND, 0);
  m_types.Add(propTEXT);
  m_controls.Add(ctrl);
  m_groupIds.Add(m_currGroupId);
}

void PropDlg::AddStaticTextProp(wxSizer* sizer, wxString text, wxString defValue)
{
  if (!sizer) // only update value
  {
	((wxStaticText*)m_controls[m_updateIndex])->SetLabel(defValue);
	m_updateIndex++;
	return;
  }
  // add static text
  if (text.length())
	sizer->Add(new wxStaticText(this, -1, text));
  wxStaticText* ctrl = new wxStaticText(this, -1, defValue);
  sizer->Add(ctrl, 1, wxEXPAND, 0);
  m_types.Add(propSTAT_TEXT);
  m_controls.Add(ctrl);
  m_groupIds.Add(m_currGroupId);
}

void PropDlg::AddCheckProp(wxSizer* sizer, wxString text, bool defValue,
  bool readonly, int id)
{
  if (!sizer) // only update value
  {
	wxCheckBox* ctrl = (wxCheckBox*)m_controls[m_updateIndex];
	ctrl->SetValue(defValue);
	ctrl->Enable(!readonly);
	m_updateIndex++;
	return;
  }
  // add checkbox control
  wxCheckBox* ctrl = new wxCheckBox(this, id, wxEmptyString);
  ctrl->SetValue(defValue);
  ctrl->Enable(!readonly);
  ctrl->SetName(wxString::Format(_T("%d"), m_controls.GetCount()));
  sizer->Add(ctrl, 1, wxEXPAND, 0);
  m_types.Add(propCHECK);
  m_controls.Add(ctrl);
  m_groupIds.Add(m_currGroupId);
}

void PropDlg::AddComboProp(wxSizer* sizer, const wxString& text,
  const wxString& defValue, const wxArrayString& choices, long style)
{
  if (!sizer) // only update value
  {
	wxComboBox* ctrl = (wxComboBox*)m_controls[m_updateIndex];
	ctrl->Clear();
	SetComboData(ctrl, defValue, choices);
	m_updateIndex++;
	return;
  }
  // add combobox control
  if (text.length())
	sizer->Add(new wxStaticText(this, -1, text), 0, 0, 0);
  wxString strings[1];
  wxComboBox* ctrl = new wxComboBox(this, -1, defValue,
    wxDefaultPosition, wxDefaultSize, 0, strings, style);
  SetComboData(ctrl, defValue, choices);
  sizer->Add(ctrl, 1, wxEXPAND, 0);
  m_types.Add(propCOMBO);
  m_controls.Add(ctrl);
}

void PropDlg::SetComboData(wxComboBox* ctrl, const wxString& defValue,
  const wxArrayString& choices)
{
  int sel = -1;
  for (int i=0; i<(int)choices.GetCount(); i++)
  {
	ctrl->Append(choices[i]);
	if (choices[i] == defValue)
	  sel = i;
  }
  ctrl->SetValue(defValue);
  ctrl->SetSelection(sel);
}

//--- Grid property ---//
void PropDlg::AddGridProp(wxSizer* sizer, const wxArrayPtrVoid& data,
  const wxString& rowTitle, bool editable)
{
  if (!sizer) // only update value
  {
	wxGrid* grid = (wxGrid*) m_controls[m_updateIndex];
	grid->ClearGrid();
	SetGridData(grid, data, rowTitle, editable);
	m_updateIndex++;
	return;
  }
  wxGrid* grid = new wxGrid(this, -1, wxPoint( 0, 0 ), wxSize(100, 100));
  grid->CreateGrid(0,0);
  SetGridData(grid, data, rowTitle, editable);

  // todo: autosize RowLableSize
  int width = 50;
  grid->SetRowLabelSize(50);  
  //grid->SetRowSize(0, 60 );
  //grid->SetColSize(0, 120);
  //grid->AutoSize();
  grid->Fit();
  wxArrayString* cols = (wxArrayString*) data.Item(0);
  for (int c = 0; c<(int)cols->GetCount(); c++)
  	width += grid->GetColSize(c);
  grid->SetSize(width+20,100);

  sizer->Add(grid, 1, wxEXPAND);
  m_types.Add(propGRID);
  m_controls.Add(grid);
}

void PropDlg::SetGridData(wxGrid* grid, const wxArrayPtrVoid& data,
  const wxString& rowTitle, bool editable)
{
  grid->EnableEditing(editable);

  wxArrayString* cols = (wxArrayString*) data.Item(0);
  int cnt = cols->GetCount()-grid->GetNumberCols();
  if (cnt>0)
    grid->AppendCols(cnt);
  else if (cnt<0)
	grid->DeleteCols(0, -cnt);
  grid->SetColLabelSize(grid->GetDefaultRowSize());
  for (int c = 0; c<(int)cols->GetCount(); c++)
    grid->SetColLabelValue(c, cols->Item(c));
  
  cnt = data.GetCount()-(editable?0:1)-grid->GetNumberRows();
  if (cnt>0)
    grid->AppendRows(cnt);
  else if (cnt<0)
	grid->DeleteRows(0, -cnt);
  for (int r = 1; r<=(int)grid->GetNumberRows(); r++)
  {
	grid->SetRowLabelValue(r-1, rowTitle + wxString::Format(_T("%d"), r));
	if (r < (int)data.GetCount())
	{
	  wxArrayString* row = (wxArrayString*) data.Item(r);
      for (int c = 0; c<(int)cols->GetCount(); c++)
        grid->SetCellValue(r-1, c, row->Item(c));
	}
  }
}

void PropDlg::OnCellLeftClick(wxGridEvent& event)
{
  wxGrid* grid = (wxGrid*)event.GetEventObject();
  grid->ClearSelection();
  grid->SetGridCursor(event.GetRow(), event.GetCol());
  grid->EnableCellEditControl();
}

void PropDlg::OnCellChange(wxGridEvent& event)
{
  wxGrid* grid = (wxGrid*)event.GetEventObject();
  if (event.GetRow() == grid->GetNumberRows()-1)
    grid->AppendRows();
  wxString rowTitle = grid->GetRowLabelValue(0);
  rowTitle = rowTitle.Mid(0, rowTitle.length()-1);
  grid->SetRowLabelValue(grid->GetNumberRows()-1,
	rowTitle + wxString::Format(_T("%d"), grid->GetNumberRows()));
}

void PropDlg::OnCellRightClick(wxGridEvent& event)
{
  wxGrid* grid = (wxGrid*)event.GetEventObject();
  if (grid->IsEditable())
  {
	m_currObject = grid;
	m_currObjectItem = event.GetRow();
	wxMenu* menu = new wxMenu;
    menu->Append(ROW_DELETE_ID, _("&Delete"));
	menu->FindItemByPosition(0)->Enable(event.GetRow()<grid->GetNumberRows()-1);
	grid->PopupMenu(menu, event.GetPosition());
  }
}

void PropDlg::OnRowDelete(wxCommandEvent& event)
{
  wxGrid* grid = (wxGrid*)m_currObject;
  grid->DeleteRows(m_currObjectItem);

}

//--- Font property ---//
void PropDlg::AddFontProp(wxSizer* sizer, wxString text, wxFont font)
{
  wxFontData data;
  data.SetInitialFont(font);
  AddFontProp(sizer, text, data);
}

void PropDlg::AddFontProp(wxSizer* sizer, wxString text, wxFontData font)
{
  if (!sizer) // only update value
  {
	delete (wxFontData*)m_controls[m_updateIndex];
	m_controls[m_updateIndex] = new wxFontData(font);
	m_updateIndex++;
	return;
  }
  if (text.length())
	sizer->Add(new wxStaticText(this, -1, text), 0, 0, 0);
  wxButton* ctrl = new wxButton(this, SELECT_FONT_BT_ID, _T("..."));
  ctrl->SetClientSize(24, 24);
  ctrl->SetName(wxString::Format(_T("%d"), m_controls.GetCount()));
  sizer->Add(ctrl, 0, 0, 0);
  m_types.Add(propFONT);
  m_controls.Add(new wxFontData(font));
  m_groupIds.Add(m_currGroupId);
}

void PropDlg::OnSelectFont(wxCommandEvent& event)
{
  long index = -1;
  ((wxButton*)event.GetEventObject())->GetName().ToLong(&index);
  wxFontData* data = (wxFontData*)m_controls[index];
  wxFontDialog dialog(this, data);
  if (dialog.ShowModal() == wxID_OK)
	*data = dialog.GetFontData();
}

void PropDlg::AddColourProp(wxSizer* sizer, wxString text, wxColour colour)
{
  if (!sizer) // only update value
  {
	((ColourPanel*)m_controls[m_updateIndex])->SetColour(colour);
	m_updateIndex++;
	return;
  }
  if (text.length())
	sizer->Add(new wxStaticText(this, -1, text), 0, 0, 0);
  wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
  ColourPanel* panel = new ColourPanel(this, -1);
  panel->SetSize(wxSize(20, 20));
  panel->SetColour(colour);
  sizer2->Add(panel, 1, wxALL, 2);
  wxButton* ctrl = new wxButton(this, SELECT_COLOR_BT_ID, _T("..."));
  ctrl->SetClientSize(24, 24);
  ctrl->SetName(wxString::Format(_T("%d"), m_controls.GetCount()));
  sizer2->Add(ctrl);
  sizer->Add(sizer2);
  m_types.Add(propCOLOUR);
  m_controls.Add(panel);
  m_groupIds.Add(m_currGroupId);
}

void PropDlg::OnSelectColour(wxCommandEvent& event)
{
  long index = -1;
  ((wxButton*)event.GetEventObject())->GetName().ToLong(&index);
  ColourPanel* panel = ((ColourPanel*)m_controls[index]);
  wxColourData data;
  data.SetColour(panel->GetColour());
  wxColourDialog dialog(this, &data);
  if (dialog.ShowModal() == wxID_OK)
	panel->SetColour(dialog.GetColourData().GetColour());
}

// Groups
wxSizer* PropDlg::BeginGroup(wxSizer* sizer,
  wxString title, wxString checkTitle, bool defValue, bool readonly)
{
  if (!sizer) // only update value
  {
	m_currGroupChecker = NULL;
	if (checkTitle.length())
	{
	  AddCheckProp(NULL, wxEmptyString, defValue, readonly, GROUP_CHECK_ID);
	  m_currGroupChecker = (wxCheckBox*)m_controls[m_updateIndex-1];
	}
	return NULL;
  }
  m_currGroupId++;
  wxStaticBox* sb = new wxStaticBox(this, -1, title);
  wxStaticBoxSizer* sbsizer = new wxStaticBoxSizer(sb, wxVERTICAL);
  sizer->Add(sbsizer, 0, wxEXPAND);
  if (checkTitle.length())
  {
    wxFlexGridSizer* groupSizer = new wxFlexGridSizer(2, 2, 8, 4);
    groupSizer->AddGrowableCol(1);
    sbsizer->Add(groupSizer, 0, wxEXPAND|wxALL, 4);
    AddCheckProp(groupSizer, wxEmptyString, defValue, readonly, GROUP_CHECK_ID);
    m_currGroupChecker = m_controls[m_controls.GetCount()-1];
	AddText(groupSizer, checkTitle);
	AddText(groupSizer, wxEmptyString);
    return groupSizer;
  }
  m_currGroupChecker = NULL;
  return sbsizer;
}

void PropDlg::EndGroup()
{
  m_currGroupId++;
  if (m_currGroupChecker)
  {
	wxCommandEvent evt;
	evt.SetEventObject((wxObject*)m_currGroupChecker);
	OnGroupCheck(evt);
  }
  m_currGroupChecker = NULL;
}

void PropDlg::OnGroupCheck(wxCommandEvent& event)
{
  long index = -1;
  wxCheckBox* ctrl = (wxCheckBox*)event.GetEventObject();
  ctrl->GetName().ToLong(&index);
  int groupId = m_groupIds[index];
  for (int i = 0; i<(int)m_groupIds.GetCount(); i++)
    if (i != index && m_groupIds[i] == groupId && m_types[i] != propFONT)
	  ((wxWindow*) m_controls[i])->Enable(ctrl->GetValue());
}

///////////////////////// Get Values /////////////////////////////////////////

wxString PropDlg::GetString(int index)
{
  switch(m_types[index])
  {
    case propTEXT:
      return ((wxTextCtrl*)m_controls[index])->GetValue();
    case propSTAT_TEXT:
	  return ((wxStaticText*)m_controls[index])->GetLabel();
    case propCOMBO:
	  return ((wxComboBox*)m_controls[index])->GetValue();
  }
  return wxEmptyString;
}

int PropDlg::GetInt(int index)
{
  switch(m_types[index])
  {
    case propCOMBO:
      return ((wxComboBox*)m_controls[index])->GetSelection();
  }
  return -1;
}

bool PropDlg::GetBool(int index)
{
  switch(m_types[index])
  {
    case propCHECK:
      return ((wxCheckBox*)m_controls[index])->GetValue();
  }
  return true;
}

wxArrayPtrVoid PropDlg::GetGrid(int index)
{
  wxArrayPtrVoid data;
  if (m_types[index] != propGRID)
	return data;
  wxGrid* grid = (wxGrid*)m_controls[index];
  grid->SaveEditControlValue();
  int count = grid->GetNumberRows() - (grid->IsEditable() ? 1 : 0);
  for (int r = 0; r<count; r++)
  {
    wxArrayString* row = new wxArrayString;
    for (int c = 0; c<(int)grid->GetNumberCols(); c++)
      row->Add(grid->GetCellValue(r,c));
	data.Add(row);
  }
  return data;
}

wxFontData PropDlg::GetFont(int index)
{
  if (m_types[index] != propFONT)
	return wxFontData();
  wxFontData* data = (wxFontData*)m_controls[index];
  if (!data->GetChosenFont().Ok())
	data->SetChosenFont(data->GetInitialFont());
  return *data;
}

wxColour PropDlg::GetColour(int index)
{
  if (m_types[index] != propCOLOUR)
	return wxColour();
  return ((ColourPanel*)m_controls[index])->GetColour();
}

